import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Type } from '@angular/core';
import { fakeAsync, tick } from '@angular/core/testing';
import { AbstractControl, FormBuilder } from '@angular/forms';
import { hot } from 'jest-marbles';
import { singleCold, singleHot } from 'libs/tech-shared/test/marbles';
import { Observable, lastValueFrom, throwError } from 'rxjs';
import { createOrganisationseinheit, createOrganisationseinheitError } from '../../../../../test/user/user';
import { Organisationseinheit, OrganisationseinheitError, OrganisationseinheitErrorType } from '../../../user/user.model';
import { getOrganisationseinheitErrorMessage } from '../../../user/user.util';
import { OrganisationseinheitService } from '../../organisationseinheit.service';
import { OrganisationseinheitFormService } from './organisationseinheit-form.service';

describe('OrganisationseinheitFormService', () => {
  let formService: OrganisationseinheitFormService;
  let organisationseinheit: Organisationseinheit;
  const organisationseinheitService: Mock<OrganisationseinheitService> = mockResourceService(OrganisationseinheitService);

  const formBuilder: FormBuilder = new FormBuilder();

  beforeEach(() => {
    formService = new OrganisationseinheitFormService(formBuilder, useFromMock(organisationseinheitService));
    organisationseinheit = createOrganisationseinheit();
    formService.form.setValue({
      [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD]: organisationseinheit.name,
      [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD]: organisationseinheit.organisationseinheitIds.join(','),
    });
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('submit', () => {
    beforeEach(() => {
      formService.handleError = jest.fn();
    });

    describe('with successful validation', () => {
      beforeEach(() => {
        formService.validate = jest.fn().mockReturnValue(true);
      });
      it('should call handle error with service call observable', fakeAsync(() => {
        const error: OrganisationseinheitError = createOrganisationseinheitError();
        formService.callService = jest.fn().mockReturnValue(throwError(() => error));

        formService.submit().subscribe();
        tick();

        expect(formService.handleError).toHaveBeenCalledWith(error);
      }));

      it('should emit emit progress as false on error', () => {
        const error: OrganisationseinheitError = createOrganisationseinheitError();
        formService.callService = jest.fn().mockReturnValue(hot('a#', { a: true }, error));

        const progressObservable: Observable<boolean> = formService.submit();

        expect(progressObservable).toBeObservable(hot('a(b|)', { a: true, b: false }));
      });

      it('should return progress observable', () => {
        formService.callService = jest.fn().mockReturnValue(singleCold(true));

        const progressObservable: Observable<boolean> = formService.submit();

        expect(progressObservable).toBeObservable(singleCold(true));
      });
    });
    describe('with unsuccessful validation', () => {
      beforeEach(() => {
        formService.validate = jest.fn().mockReturnValue(false);
      });

      it('should return progress observable with false', async () => {
        const progressObservable: Observable<boolean> = formService.submit();

        const progress: boolean = await lastValueFrom(progressObservable);

        expect(progress).toBeFalsy();
      });
    });
  });

  describe('validate', () => {
    const hasIdMissingError = () =>
      formService.form.hasError(
        OrganisationseinheitErrorType.ID_MISSING,
        OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD,
      );
    describe('without organisationeinheitIds', () => {
      beforeEach(() => {
        formService.form.setValue({
          [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD]: organisationseinheit.name,
          [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD]: ',',
        });
      });

      it('should set id-missing error', () => {
        formService.validate();

        expect(hasIdMissingError()).toBeTruthy();
      });
      it('should be invalid', () => {
        const valid: boolean = formService.validate();

        expect(valid).toBeFalsy();
      });
    });
    describe('with organisationeinheitIds', () => {
      it('should not set error', () => {
        formService.validate();

        expect(hasIdMissingError()).toBeFalsy();
      });

      it('should be valid', () => {
        const valid: boolean = formService.validate();

        expect(valid).toBeTruthy();
      });
    });
  });

  describe('call service', () => {
    it('should use create if is not patch', () => {
      formService.isPatch = jest.fn().mockReturnValue(false);
      formService.create = jest.fn().mockReturnValue(singleHot(true));

      const progressObservable: Observable<boolean> = formService.callService();

      expect(progressObservable).toBeObservable(singleHot(true));
    });

    it('should use save if is patch', () => {
      formService.isPatch = jest.fn().mockReturnValue(true);
      formService.save = jest.fn().mockReturnValue(singleHot(true));

      const progressObservable: Observable<boolean> = formService.callService();

      expect(progressObservable).toBeObservable(singleHot(true));
    });
  });

  describe('is patch', () => {
    it('should return false without source', () => {
      formService.source = null;

      const isPatch: boolean = formService.isPatch();

      expect(isPatch).toBeFalsy();
    });
    it('should return true with source', () => {
      formService.source = createOrganisationseinheit();

      const isPatch: boolean = formService.isPatch();

      expect(isPatch).toBeTruthy();
    });
  });

  describe('create', () => {
    it('should call create organisationseinheit', () => {
      formService.create();

      expect(organisationseinheitService.create).toHaveBeenCalledWith({
        name: organisationseinheit.name,
        organisationseinheitIds: organisationseinheit.organisationseinheitIds,
      });
    });

    it('should call create organisationseinheit with empty form', () => {
      formService.form.setValue({
        [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD]: null,
        [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD]: null,
      });
      formService.create();

      expect(organisationseinheitService.create).toHaveBeenCalledWith({
        name: '',
        organisationseinheitIds: [],
      });
    });

    it('should return progress observable', () => {
      organisationseinheitService.create.mockReturnValue(singleCold(true));

      const progressObservable: Observable<boolean> = formService.create();

      expect(progressObservable).toBeObservable(singleCold(true));
    });
  });

  describe('save', () => {
    it('should call save organisationseinheit', () => {
      formService.source = createOrganisationseinheit();

      formService.save();

      expect(organisationseinheitService.save).toHaveBeenCalledWith({
        id: formService.source.id,
        name: organisationseinheit.name,
        organisationseinheitIds: organisationseinheit.organisationseinheitIds,
      });
    });

    it('should call save organisationseinheit with empty form', () => {
      formService.source = createOrganisationseinheit();
      formService.form.setValue({
        [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD]: null,
        [OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD]: null,
      });
      formService.save();

      expect(organisationseinheitService.save).toHaveBeenCalledWith({
        id: formService.source.id,
        name: '',
        organisationseinheitIds: [],
      });
    });

    it('should return progress observable', () => {
      organisationseinheitService.save.mockReturnValue(singleCold(true));

      const progressObservable: Observable<boolean> = formService.save();

      expect(progressObservable).toBeObservable(singleCold(true));
    });
  });

  describe('handle error', () => {
    it.each([OrganisationseinheitErrorType.NAME_CONFLICT, OrganisationseinheitErrorType.NAME_MISSING])(
      'should set error on name field on %s',
      (type: OrganisationseinheitErrorType) => {
        const error: OrganisationseinheitError = {
          ...createOrganisationseinheitError(),
          errorType: type,
        };

        formService.handleError(error);
        const errorMessage = formService.form.getError(type, OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD);
        expect(errorMessage).toEqual(getOrganisationseinheitErrorMessage(error));
      },
    );

    it('should not set error on name field for unknown errors', () => {
      const unknownError: OrganisationseinheitError = createOrganisationseinheitError(undefined);

      formService.handleError(unknownError);

      expect(formService.form.errors).toBeNull();
    });
  });

  describe('patch', () => {
    const organisationseinheit: Organisationseinheit = createOrganisationseinheit();

    it('should reset', () => {
      formService.reset = jest.fn();

      formService.patch(organisationseinheit);

      expect(formService.reset).toHaveBeenCalled();
    });

    it('should set source', () => {
      formService.patch(organisationseinheit);

      expect(formService.source).toBe(organisationseinheit);
    });

    it('should set name', () => {
      formService.patch(organisationseinheit);

      const formControl: AbstractControl = formService.form.get(OrganisationseinheitFormService.ORGANISATIONSEINHEIT_NAME_FIELD);
      expect(formControl.value).toEqual(organisationseinheit.name);
    });

    it('should set organisationseinheitIds', () => {
      formService.patch(organisationseinheit);

      const formControl: AbstractControl = formService.form.get(OrganisationseinheitFormService.ORGANISATIONSEINHEIT_IDS_FIELD);
      expect(formControl.value).toEqual(organisationseinheit.organisationseinheitIds.join(', '));
    });
  });

  describe('split organisationseinheitIds by comma', () => {
    it('should return', () => {
      const organisationseinheitIds: string[] = formService.splitOrganisationseinheitIds('123, 555 ,  666');

      expect(organisationseinheitIds).toEqual(['123', '555', '666']);
    });

    it('should filter empty organisationseinheitIds', () => {
      const organisationseinheitIds: string[] = formService.splitOrganisationseinheitIds(',55,,66,');

      expect(organisationseinheitIds).toEqual(['55', '66']);
    });
  });

  describe('reset', () => {
    it('should set source to null', () => {
      formService.form.reset = jest.fn();

      formService.reset();

      expect(formService.source).toBeNull();
    });

    it('should call form reset', () => {
      formService.form.reset = jest.fn();

      formService.reset();

      expect(formService.form.reset).toHaveBeenCalled();
    });

    it('should mark as untouched', () => {
      formService.patch(organisationseinheit);

      expect(formService.form.untouched).toBeTruthy();
    });
  });

  describe('is invalid', () => {
    it('should return true', () => {
      formService.form.setErrors({ some: 'message' });

      const invalid: boolean = formService.isInvalid();

      expect(invalid).toBeTruthy();
    });

    it('should return false', () => {
      const invalid: boolean = formService.isInvalid();

      expect(invalid).toBeFalsy();
    });
  });
});

function mockResourceService<T>(service: Type<T>): Mock<T> {
  return <Mock<T>>{ ...mock(service), create: jest.fn(), save: jest.fn() };
}
