/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { CommandOrder, CommandResource } from '@alfa-client/command-shared';
import { FormatToPrettyDatePipe } from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { MockComponent } from 'ng-mocks';
import { ExpansionPanelWithUserComponent } from '../expansion-panel-with-user/expansion-panel-with-user.component';
import { HistorieItemAttachmentComponent } from '../historie-item-attachment/historie-item-attachment.component';
import { HistorieItemHeaderComponent } from '../historie-item-header/historie-item-header.component';
import { HistorieItemWiedervorlageStatusComponent } from './historie-item-wiedervorlage-status/historie-item-wiedervorlage-status.component';
import { HistorieItemWiedervorlageComponent } from './historie-item-wiedervorlage.component';

registerLocaleData(localeDe);

describe('HistorieItemWiedervorlageComponent', () => {
  let component: HistorieItemWiedervorlageComponent;
  let fixture: ComponentFixture<HistorieItemWiedervorlageComponent>;

  const wiedervorlageCommand: CommandResource = {
    ...createCommandResource(),
    order: CommandOrder.CREATE_WIEDERVORLAGE,
    body: {
      item: { ...createWiedervorlageResource() },
    },
  };

  const userExpansionPanel: string = getDataTestClassOf(
    'historie-item-wiedervorlage-user-expansion-panel',
  );
  const itemHeadline: string = getDataTestClassOf('historie-item-wiedervorlage-header');
  const attachment: string = getDataTestClassOf('historie-item-wiedervorlage-attachment');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        FormatToPrettyDatePipe,
        HistorieItemWiedervorlageComponent,
        MockComponent(HistorieItemHeaderComponent),
        MockComponent(ExpansionPanelWithUserComponent),
        MockComponent(HistorieItemWiedervorlageStatusComponent),
        MockComponent(HistorieItemAttachmentComponent),
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(HistorieItemWiedervorlageComponent);
    component = fixture.componentInstance;
    component.command = wiedervorlageCommand;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should set headline for ' + CommandOrder.CREATE_WIEDERVORLAGE, () => {
      component.command = { ...wiedervorlageCommand, order: CommandOrder.CREATE_WIEDERVORLAGE };

      component.ngOnInit();

      expect(component.headline).toBe('eine Wiedervorlage hinzugefügt.');
    });

    it('should set headline by %s order' + CommandOrder.EDIT_WIEDERVORLAGE, () => {
      component.command = { ...wiedervorlageCommand, order: CommandOrder.EDIT_WIEDERVORLAGE };

      component.ngOnInit();

      expect(component.headline).toBe('eine Wiedervorlage bearbeitet.');
    });

    it('should set headline by %s order' + CommandOrder.WIEDERVORLAGE_ERLEDIGEN, () => {
      component.command = { ...wiedervorlageCommand, order: CommandOrder.WIEDERVORLAGE_ERLEDIGEN };

      component.ngOnInit();

      expect(component.headline).toBe('eine Wiedervorlage als erledigt markiert.');
    });

    it('should set headline by %s order' + CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN, () => {
      component.command = {
        ...wiedervorlageCommand,
        order: CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN,
      };

      component.ngOnInit();

      expect(component.headline).toBe('eine Wiedervorlage als offen markiert.');
    });
  });

  describe('has item body', () => {
    it.each([CommandOrder.WIEDERVORLAGE_ERLEDIGEN, CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN])(
      'should return false on order %s',
      (order: CommandOrder) => {
        component.command = { ...createCommandResource(), order };

        const hasItemBody: boolean = component.hasItemBody();

        expect(hasItemBody).toBeFalsy();
      },
    );

    it.each([CommandOrder.CREATE_WIEDERVORLAGE, CommandOrder.EDIT_WIEDERVORLAGE])(
      'should return true on order %s',
      (order: CommandOrder) => {
        component.command = { ...createCommandResource(), order };

        const hasItemBody: boolean = component.hasItemBody();

        expect(hasItemBody).toBeTruthy();
      },
    );
  });

  describe('show item body', () => {
    describe('on create/edit order', () => {
      beforeEach(() => {
        component.command = {
          ...createCommandResource(),
          order: CommandOrder.CREATE_WIEDERVORLAGE,
          body: { item: createWiedervorlageResource() },
        };
      });

      it('should show expansion panel', () => {
        component.ngOnInit();
        fixture.detectChanges();

        const userPanel = getElementFromFixture(fixture, userExpansionPanel);
        expect(userPanel).toBeInstanceOf(HTMLElement);
      });
      it('should hide header only', () => {
        component.ngOnInit();
        fixture.detectChanges();

        const userPanel = getElementFromFixture(fixture, itemHeadline);
        expect(userPanel).not.toBeInstanceOf(HTMLElement);
      });

      describe('attachments', () => {
        it('should hide of NOT exists', () => {
          const item = createWiedervorlageResource();
          delete item['attachments'];
          component.command = {
            ...createCommandResource(),
            order: CommandOrder.CREATE_WIEDERVORLAGE,
            body: { item },
          };
          fixture.detectChanges();

          const attachmentElement = getElementFromFixture(fixture, attachment);

          expect(attachmentElement).not.toBeInstanceOf(HTMLElement);
        });

        it('should show if exists', () => {
          component.command = {
            ...createCommandResource(),
            order: CommandOrder.CREATE_WIEDERVORLAGE,
            body: { item: { ...createWiedervorlageResource(), attachment: '' } },
          };
          fixture.detectChanges();

          const attachmentElement = getElementFromFixture(fixture, attachment);

          expect(attachmentElement).toBeInstanceOf(HTMLElement);
        });
      });
    });

    describe('on erledigen/wiedereroeffnen order', () => {
      beforeEach(() => {
        component.command = {
          ...createCommandResource(),
          order: CommandOrder.WIEDERVORLAGE_ERLEDIGEN,
          body: { item: createWiedervorlageResource() },
        };
      });

      it('should hide expansion panel', () => {
        component.ngOnInit();
        fixture.detectChanges();

        const userPanel = getElementFromFixture(fixture, userExpansionPanel);
        expect(userPanel).not.toBeInstanceOf(HTMLElement);
      });
      it('should show header only', () => {
        component.ngOnInit();
        fixture.detectChanges();

        const userPanel = getElementFromFixture(fixture, itemHeadline);
        expect(userPanel).toBeInstanceOf(HTMLElement);
      });
    });

    describe('on non body order', () => {
      beforeEach(() => {
        component.command = {
          ...createCommandResource(),
          order: CommandOrder.WIEDERVORLAGE_ERLEDIGEN,
        };
      });

      it('should hide expansion panel', () => {
        component.ngOnInit();
        fixture.detectChanges();

        const userPanel = getElementFromFixture(fixture, userExpansionPanel);
        expect(userPanel).not.toBeInstanceOf(HTMLElement);
      });
      it('should show header only', () => {
        component.ngOnInit();
        fixture.detectChanges();

        const userPanel = getElementFromFixture(fixture, itemHeadline);
        expect(userPanel).toBeInstanceOf(HTMLElement);
      });
    });
  });
});
