/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

import { faker } from '@faker-js/faker';
import { Resource } from '@ngxp/rest';
import { createDummyResource } from 'libs/tech-shared/test/resource';
import { LinkRelationName } from './resource/resource.model';
import {
  EMPTY_STRING,
  allEmpty,
  convertForDataTest,
  convertToBoolean,
  decodeUrlFromEmbedding,
  encodeUrlForEmbedding,
  getFirstLetter,
  getStringValue,
  hasAnyLink,
  hasContent,
  hasMinLength,
  isNotEmpty,
  isNotNil,
  isNotNull,
  notHasAnyLink,
  replaceAllWhitespaces,
  replacePlaceholder,
  replacePlaceholders,
  simpleTransliteration,
} from './tech.util';

describe('TechUtil', () => {
  describe('replaceAllWhitespaces', () => {
    it('should replace whitespaces in value', () => {
      const value: string = 'value to replace whitespaces';

      const result: string = replaceAllWhitespaces(value, '_');

      expect(result).toBe('value_to_replace_whitespaces');
    });
  });

  describe('simpleTransliteration', () => {
    it('should replace whitespaces in value', () => {
      const value: string = 'áàâäãéèëêíìïîóòöôõúùüûñçăşţ-_';

      const result: string = simpleTransliteration(value);

      expect(result).toBe('aaaaaeeeeiiiiooooouuuuncast-_');
    });
  });

  describe('replace placeholder', () => {
    it('should replace with value', () => {
      const text = 'replace {this}';
      const placeholder = 'this';
      const value = 'with this';

      const replaced = replacePlaceholder(text, placeholder, value);

      expect(replaced).toBe('replace with this');
    });

    it('should replace all', () => {
      const text = 'replace {here} and {here}';
      const placeholder = 'here';
      const value = 'there';

      const replaced = replacePlaceholder(text, placeholder, value);

      expect(replaced).toBe('replace there and there');
    });

    it('should ignore other placeholder', () => {
      const text = 'replace {here}';
      const placeholder = 'there';
      const value = 'this';

      const replaced = replacePlaceholder(text, placeholder, value);

      expect(replaced).toBe('replace {here}');
    });
  });

  describe('replace placeholders', () => {
    it('should replace all placeholder', () => {
      const text = 'replace {here} and {there}';
      const placeholders = { here: 'this', there: 'that' };

      const replaced = replacePlaceholders(text, placeholders);

      expect(replaced).toBe('replace this and that');
    });
  });

  describe('get first letter', () => {
    it('should return on valid string value', () => {
      const value: string = `e ${faker.random.word()}`;

      expect(getFirstLetter(value)).toBe('e');
    });

    it('should return undefined on undefined value', () => {
      expect(getFirstLetter(undefined)).toBeNull();
    });

    it('should return undefined on null value', () => {
      expect(getFirstLetter(null)).toBeNull();
    });
  });

  describe('isNotEmpty', () => {
    it('should return true on filled string', () => {
      const result: boolean = isNotEmpty(faker.random.word());

      expect(result).toBeTruthy();
    });

    it('should return true if array contains elements', () => {
      const result: boolean = isNotEmpty(faker.helpers.arrayElement());

      expect(result).toBeTruthy();
    });

    it('should return true if object contains properties', () => {
      const result: boolean = isNotEmpty(faker.helpers.objectValue({ key: 'value' }));

      expect(result).toBeTruthy();
    });

    it('should return false if string length is zero', () => {
      const result: boolean = isNotEmpty('');

      expect(result).toBeFalsy();
    });

    it('should return false if array length is zero', () => {
      const result: boolean = isNotEmpty([]);

      expect(result).toBeFalsy();
    });

    it('should return false if object length is zero', () => {
      const result: boolean = isNotEmpty({});

      expect(result).toBeFalsy();
    });
  });

  describe('isNotNil', () => {
    it('should return true if value is filled string', () => {
      const result: boolean = isNotNil(faker.random.word());

      expect(result).toBeTruthy();
    });

    it('should return false if value is null', () => {
      const result: boolean = isNotNil(null);

      expect(result).toBeFalsy();
    });

    it('should return false if value is undefined', () => {
      const result: boolean = isNotNil(undefined);

      expect(result).toBeFalsy();
    });
  });

  describe('isNotNulll', () => {
    it('should return true if value is filled string', () => {
      const result: boolean = isNotNull(faker.random.word());

      expect(result).toBeTruthy();
    });

    it('should return false if value is null', () => {
      const result: boolean = isNotNull(null);

      expect(result).toBeFalsy();
    });

    it('should return true if value is undefined', () => {
      const result: boolean = isNotNull(undefined);

      expect(result).toBeTruthy();
    });
  });

  describe('hasMinLength with length  of 3', () => {
    const minValue: number = 3;
    const SEARCH_STRING = 'i search for...';

    it('should return false on null value', () => {
      const hasLength = hasMinLength(null, minValue);

      expect(hasLength).toBeFalsy();
    });

    it('should return false on undefined value', () => {
      const hasLength = hasMinLength(undefined, minValue);

      expect(hasLength).toBeFalsy();
    });

    it('should return false on empty string => ""', () => {
      const hasLength = hasMinLength(EMPTY_STRING, minValue);

      expect(hasLength).toBeFalsy();
    });

    it('should return false on length less than 2', () => {
      const hasLength = hasMinLength(EMPTY_STRING, minValue);

      expect(hasLength).toBeFalsy();
    });

    it('should return true on length equals than 3 ', () => {
      const hasLength = hasMinLength('huh', minValue);

      expect(hasLength).toBeTruthy();
    });

    it('should return true on length greater than 3 ', () => {
      const hasLength = hasMinLength(SEARCH_STRING, minValue);

      expect(hasLength).toBeTruthy();
    });
  });

  describe('convertForDataTest', () => {
    it('should replace whitespaces and transliterate Umlauts in value', () => {
      const value: string = 'value to replace whitespaces and Ümläuts with Öös';

      const result: string = convertForDataTest(value);

      expect(result).toBe('value_to_replace_whitespaces_and_Umlauts_with_Oos');
    });
  });

  describe('get string value', () => {
    it('should return empty string on null', () => {
      const value: string = null;

      const result: string = getStringValue(value);

      expect(result).toBe(EMPTY_STRING);
    });

    it('should return empty string on undefined', () => {
      const value: string = undefined;

      const result: string = getStringValue(value);

      expect(result).toBe(EMPTY_STRING);
    });

    it('should return given value', () => {
      const value: string = 'value of this...';

      const result: string = getStringValue(value);

      expect(result).toBe(value);
    });
  });

  describe('encodeUrlForEmbedding', () => {
    it('should return encoded URL to be embedded in a URI', () => {
      const url: string = faker.internet.url();
      const forbiddenChars: RegExp = /[^-a-zA-Z0-9_=]/;

      const encoded: string = encodeUrlForEmbedding(url);
      const searchForbiddenChars: number = encoded.search(forbiddenChars);

      expect(searchForbiddenChars).toBe(-1);
      expect(url).toBe(decodeUrlFromEmbedding(encoded));
    });
  });

  describe('decodeUrlFromEmbedding', () => {
    it('should return URL from encoded string', () => {
      const url: string = faker.internet.url();
      const encodedUrl: string = encodeUrlForEmbedding(url);

      const decoded: string = decodeUrlFromEmbedding(encodedUrl);

      expect(decoded).toBe(url);
    });
  });

  describe('allEmpty', () => {
    it('should return true on all empty', () => {
      const empty: boolean = allEmpty(EMPTY_STRING, EMPTY_STRING, EMPTY_STRING);

      expect(empty).toBeTruthy();
    });

    it('should return false on one NOT empty', () => {
      const empty: boolean = allEmpty(EMPTY_STRING, EMPTY_STRING, 'quatsch');

      expect(empty).toBeFalsy();
    });

    it('should return false on all NOT empty', () => {
      const empty: boolean = allEmpty('1', '2', '3');

      expect(empty).toBeFalsy();
    });

    it('should return true on null', () => {
      const empty: boolean = allEmpty(null);

      expect(empty).toBeTruthy();
    });

    it('should return true on undefined', () => {
      const empty: boolean = allEmpty(undefined);

      expect(empty).toBeTruthy();
    });
  });

  describe('hasContent', () => {
    it.each(['a', ' a '])('should return true for %s', (content: string) => {
      expect(hasContent(content)).toBeTruthy();
    });

    it.each(['', ' ', null])('should return false for %s', (content: string) => {
      expect(hasContent(content)).toBeFalsy();
    });
  });

  describe('convertToBoolean', () => {
    it('should return true', () => {
      const value = convertToBoolean('true');

      expect(value).toBeTruthy();
    });

    it('should return false', () => {
      const value = convertToBoolean('false');

      expect(value).toBeFalsy();
    });

    it('should return undefined on non matching string', () => {
      const value = convertToBoolean('quatsch');

      expect(value).toBeUndefined();
    });
  });

  describe('has any link', () => {
    const oneLink: string = 'one';
    const anotherLink: string = 'another';
    const resource: Resource = createDummyResource([oneLink, anotherLink]);

    it('should return true if resource has at least on link', () => {
      const result: boolean = hasAnyLink(resource, oneLink, 'notExists');

      expect(result).toBe(true);
    });

    it('resource return true if resoure has multiple links', () => {
      const result: boolean = hasAnyLink(resource, oneLink, anotherLink);

      expect(result).toBe(true);
    });

    it('should return false if resource has no of given links', () => {
      const result: boolean = hasAnyLink(resource, 'notExists', 'notExistsToo');

      expect(result).toBe(false);
    });

    it('should return false if resource is null', () => {
      const result: boolean = hasAnyLink(null, 'notExists', 'notExistsToo');

      expect(result).toBe(false);
    });
  });

  describe('not has any link', () => {
    const link: LinkRelationName = 'one';
    const otherLink: LinkRelationName = 'two';

    it('should return false if at least one link exists', () => {
      const result: boolean = notHasAnyLink(createDummyResource([link]), link, otherLink);

      expect(result).toBe(false);
    });
    it('should return true if no link exists', () => {
      const result: boolean = notHasAnyLink(createDummyResource(), link, otherLink);

      expect(result).toBe(true);
    });
  });
});
