/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import java.io.OutputStream;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.errorhandling.ResourceNotFoundException;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserProfile;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@RequiredArgsConstructor
@Log4j2
@Service
class PostfachMailService {

	private final PostfachMailRemoteService remoteService;
	private final PostfachNachrichtPdfService pdfService;
	private final BinaryFileService fileService;
	private final UserService userService;
	private final CommandService commandService;

	public PostfachMail findById(PostfachNachrichtId nachrichtId) {
		return remoteService.findById(nachrichtId)
				.orElseThrow(() -> new ResourceNotFoundException(PostfachMail.class, nachrichtId));
	}

	public Command sendPostfachMail(CreateCommand command, long relationVersion) {
		return commandService.createCommand(command, relationVersion);
	}

	// @Async FIXME Er scheint den CallContext zu verlieren
	public void resendPostfachMail(String commandId, String postfachMailId) {
		try {
			remoteService.resendPostfachMail(commandId, postfachMailId);
		} catch (RuntimeException e) {
			LOG.error("Error sending PostfachNachricht: " + e.getMessage(), e);
			// FIXME mark command as in error
		}
	}

	public OutputStream getAllAsPdf(VorgangWithEingang vorgang, OutputStream out) {
		var postfachNachrichtPdfDataList = buildPostfachNachrichtPdfDataList(vorgang.getId());

		return pdfService.getAllAsPdf(vorgang, postfachNachrichtPdfDataList, out);
	}

	Stream<PostfachNachrichtPdfData> buildPostfachNachrichtPdfDataList(String vorgangId) {
		var postfachNachrichten = getAll(vorgangId).toList();
		var ozgFileIdOzgFileMap = getFiles(postfachNachrichten).collect(Collectors.toMap(OzgFile::getId, OzgFile::getName));

		return postfachNachrichten.stream()
				.sorted(Comparator.comparing(PostfachMail::getCreatedAt).reversed())
				.map(postfachNachricht -> buildPostfachNachrichtPdfData(postfachNachricht, ozgFileIdOzgFileMap));
	}

	public Stream<PostfachMail> getAll(String vorgangId) {
		return remoteService.findPostfachMails(vorgangId);
	}

	Stream<OzgFile> getFiles(List<PostfachMail> postfachMails) {
		return fileService.getFiles(getFileIdsFromAllAttachments(postfachMails));
	}

	List<FileId> getFileIdsFromAllAttachments(List<PostfachMail> postfachMails) {
		return postfachMails.stream()
				.map(PostfachMail::getAttachments)
				.flatMap(Collection::stream)
				.toList();
	}

	PostfachNachrichtPdfData buildPostfachNachrichtPdfData(PostfachMail postfachNachricht,
			Map<FileId, String> ozgFileIdOzgFileMap) {
		return PostfachNachrichtPdfData.builder()
				.createdAt(postfachNachricht.getCreatedAt())
				.user(getUser(postfachNachricht))
				.direction(postfachNachricht.getDirection())
				.mailBody(postfachNachricht.getMailBody())
				.subject(postfachNachricht.getSubject())
				.attachmentNames(postfachNachricht.getAttachments().stream().map(ozgFileIdOzgFileMap::get).toList())
				.build();
	}

	UserProfile getUser(PostfachMail postfachNachricht) {
		var createdBy = postfachNachricht.getCreatedBy();
		if (UserManagerUrlProvider.SENT_BY_CLIENT_USER.test(postfachNachricht)) {
			return userService.getById(createdBy);
		}
		return Optional.ofNullable(createdBy)
				.map(created -> UserProfile.builder().id(created).build())
				.orElseGet(() -> null);
	}
}