/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import static de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;

class HistorieModelAssemblerTest {

	private static final String CREATED_BY = "createdBy";

	@InjectMocks
	private HistorieModelAssembler modelAssembler;
	@Mock
	private UserManagerUrlProvider userManagerUrlProvider;

	private final String COMMAND_SINGLE_PATH = "/api/histories/" + CommandTestFactory.ID;

	private UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();

	@Test
	void shouldHaveSelfLink() {
		initUserProfileUrlProvider(urlProvider);

		var model = modelAssembler.toModel(CommandTestFactory.create());

		assertThat(model.getLink(IanaLinkRelations.SELF)).isPresent().get().extracting(Link::getHref).isEqualTo(COMMAND_SINGLE_PATH);
	}

	@DisplayName("AssignedTo Link")
	@Nested
	class TestAssignedToLink {

		private final String userProfileTemplateDummy = "UserProfileTemplateDummy/%s";

		@Nested
		class TesOnConfiguredUserManager {

			@Test
			void shouldBePresentOnExistingValue() {
				when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(true);
				when(userManagerUrlProvider.getUserProfileTemplate()).thenReturn(userProfileTemplateDummy);

				var model = modelAssembler.toModel(CommandTestFactory.createBuilder()
						.body(Map.of(HistorieModelAssembler.ASSIGNED_TO_BODY_FIELD, UserProfileTestFactory.ID)).build());

				assertThat(model.getLink(HistorieModelAssembler.REL_ASSIGNED_TO)).isPresent().get().extracting(Link::getHref)
						.isEqualTo("UserProfileTemplateDummy/" + UserProfileTestFactory.ID);
			}

			@Test
			void shouldNotBePresentOnMissingValue() {
				var model = modelAssembler.toModel(CommandTestFactory.create());

				assertThat(model.getLink(HistorieModelAssembler.REL_ASSIGNED_TO)).isNotPresent();
			}
		}

		@Nested
		class TestOnNotConfiguredUserManager {

			@Test
			void shouldBePresentOnExistingValue() {
				when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(false);

				var model = modelAssembler.toModel(CommandTestFactory.createBuilder()
						.body(Map.of(HistorieModelAssembler.ASSIGNED_TO_BODY_FIELD, UserProfileTestFactory.ID)).build());

				assertThat(model.getLink(HistorieModelAssembler.REL_ASSIGNED_TO)).isNotPresent();
			}

			@Test
			void shouldNotGetTemplateFromUserManagerUrlProvider() {
				when(userManagerUrlProvider.isConfiguredForUserProfile()).thenReturn(false);

				modelAssembler.toModel(CommandTestFactory.createBuilder()
						.body(Map.of(HistorieModelAssembler.ASSIGNED_TO_BODY_FIELD, UserProfileTestFactory.ID)).build());

				verify(userManagerUrlProvider, never()).getUserProfileTemplate();
			}
		}
	}

	@DisplayName("createdBy Link")
	@Nested
	class TestCreatedByLink {
		@Test
		void shouldExistingAtUser() {
			initUserProfileUrlProvider(urlProvider);

			var model = modelAssembler.toModel(CommandTestFactory.create());

			assertThat(model.getLink(CREATED_BY)).isPresent().get().extracting(Link::getHref)
					.isEqualTo(ROOT_URL + USER_PROFILES_API_PATH + UserProfileTestFactory.ID);
		}

		@Test
		void shouldNotExistingAtSystemNotificationMAnager() {
			var model = modelAssembler
					.toModel(CommandTestFactory.createBuilder().createdBy(UserId.from(HistorieModelAssembler.SYSTEM_NOTIFICATION_MANAGER_PREFIX))
							.build());

			assertThat(model.getLink(CREATED_BY)).isNotPresent();
		}

		@Test
		void shouldNotBePresentOnNullValue() {
			var model = modelAssembler.toModel(CommandTestFactory.createBuilder().createdBy(null).build());

			assertThat(model.getLink(CREATED_BY)).isNotPresent();
		}
	}
}