/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.command.CommandRemoteService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.kommentar.KommentarCommandController.KommentarCommandByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import lombok.SneakyThrows;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class KommentarCommandITCase {

	@Autowired
	private MockMvc mockMvc;
	@MockBean
	private KommentarRemoteService remoteService;
	@MockBean
	private CommandRemoteService commandRemoteService;

	@SpyBean
	private CurrentUserService userService;

	@WithMockUser
	@Nested
	class TestCreateCommandByKommentarId {

		@BeforeEach
		void initTest() {
			when(remoteService.getById(any())).thenReturn(KommentarTestFactory.create());
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());
		}

		@SneakyThrows
		@Test
		void shouldCreateCommand() {
			doRequestByKommentarId(createValidRequestContent()).andExpect(status().isCreated());

			verify(commandRemoteService).createCommand(any());
		}

		private String createValidRequestContent() {
			return KommentarTestFactory
					.createRequestContent(CommandTestFactory.createCreateCommandBuilder()
							.order(LegacyOrder.CREATE_KOMMENTAR)
							.body(KommentarTestFactory.create())
							.build());
		}

		@WithMockUser
		@DisplayName("should return validation error")
		@Nested
		class TestValidation {

			@SneakyThrows
			@DisplayName("for null Text")
			@Test
			void createCommandWithInvalidText() {
				String content = buildContentWithText(null);

				doRequestByKommentarId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.invalidParams.length()").value(1))
						.andExpect(jsonPath("$.invalidParams[0].name").value("editKommentar.kommentar.text"))
						.andExpect(jsonPath("$.invalidParams[0].reason").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			@SneakyThrows
			@DisplayName("for empty String in Text")
			@Test
			void createcommandWithShortText() {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByKommentarId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.invalidParams[0].name").value("editKommentar.kommentar.text"));

			}

			@SneakyThrows
			@DisplayName("for invalid text should have parameter")
			@Test
			void minMaxParameter() {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByKommentarId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.invalidParams[0].constraintParameters.length()").value(2));
			}

			private String buildContentWithText(String text) {
				var kommentar = KommentarTestFactory.createBuilder().text(text).build();
				var createCommand = CommandTestFactory.createCreateCommandBuilder()
						.order(LegacyOrder.CREATE_KOMMENTAR)
						.body(kommentar)
						.build();
				return KommentarTestFactory.createRequestContent(createCommand);
			}
		}

		@SneakyThrows
		private ResultActions doRequestByKommentarId(String content) {
			return mockMvc
					.perform(post(KommentarCommandController.KOMMENTAR_COMMANDS, KommentarTestFactory.ID, KommentarTestFactory.VERSION).with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}

	@WithMockUser
	@Nested
	class TestCreateCommandByVorgangId {

		private static final long RELATION_ID_ON_CREATE = -1;

		@BeforeEach
		void initTest() {
			when(remoteService.getById(any())).thenReturn(KommentarTestFactory.create());
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());

			doReturn(UserProfileTestFactory.ID).when(userService).getUserId();
		}

		@SneakyThrows
		@Test
		void shouldCreateCommand() {
			var content = KommentarTestFactory
					.createRequestContent(CommandTestFactory.createCreateCommandBuilder()
							.order(LegacyOrder.CREATE_KOMMENTAR)
							.body(KommentarTestFactory.create()).build());
			doRequestByVorgangId(content).andExpect(status().isCreated());

			verify(commandRemoteService).createCommand(any());
		}

		@WithMockUser
		@DisplayName("should return validation error")
		@Nested
		class TestValidation {

			@SneakyThrows
			@DisplayName("for null Text")
			@Test
			void createCommandWithInvalidText() {
				String content = buildContentWithText(null);

				doRequestByVorgangId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.invalidParams.length()").value(1))
						.andExpect(jsonPath("$.invalidParams.[0].name").value("createKommentar.kommentar.text"))
						.andExpect(jsonPath("$.invalidParams.[0].reason").value(ValidationMessageCodes.FIELD_IS_EMPTY));
			}

			@SneakyThrows
			@DisplayName("for empty String in Text")
			@Test
			void createcommandWithShortText() {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByVorgangId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.invalidParams.[0].name").value("createKommentar.kommentar.text"));

			}

			@SneakyThrows
			@DisplayName("for invalid text should have parameter")
			@Test
			void minMaxParameter() {
				String content = buildContentWithText(StringUtils.EMPTY);

				doRequestByVorgangId(content).andExpect(status().isUnprocessableEntity())
						.andExpect(jsonPath("$.invalidParams[0].constraintParameters.length()").value(2));
			}

			private String buildContentWithText(String text) {
				var kommentar = KommentarTestFactory.createBuilder().text(text).build();
				var createCommand = CommandTestFactory.createCreateCommandBuilder()
						.order(LegacyOrder.CREATE_KOMMENTAR)
						.body(kommentar)
						.build();
				return KommentarTestFactory.createRequestContent(createCommand);
			}
		}

		@SneakyThrows
		private ResultActions doRequestByVorgangId(String content) {
			return mockMvc.perform(post(KommentarCommandByVorgangController.KOMMENTAR_COMMANDS_BY_VORGANG, VorgangHeaderTestFactory.ID,
					RELATION_ID_ON_CREATE).with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}
}