/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static de.ozgcloud.alfa.vorgang.EingangTestFactory.*;
import static org.assertj.core.api.AssertionsForClassTypes.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.MethodSource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.common.clientattribute.ClientAttributeService;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.statistic.StatisticController;
import de.ozgcloud.alfa.statistic.StatisticTestFactory;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import lombok.SneakyThrows;

class VorgangControllerTest {

	private static final String RESET_POSTFACH_NACHRICHT_PATH = VorgangController.PATH + "/{0}/hasNewPostfachNachricht";
	private static final String PATH = VorgangController.PATH + "/";

	@Spy
	@InjectMocks
	private VorgangController controller;
	@Mock
	private VorgangService vorgangService;
	@Mock
	private VorgangModelAssembler modelAssembler;

	@Mock
	private ClientAttributeService clientAttributeService;

	@Mock
	private StatisticController statisticController;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("VorgangList")
	@Nested
	class TestVorgangList {

		private static final int PAGE = 5;
		private static final int LIMIT = 7;

		@Captor
		private ArgumentCaptor<FindVorgaengeHeaderRequestCriteria> criteriaCaptor;

		@BeforeEach
		void initTest() {
			when(vorgangService.findVorgaengeHeader(any())).thenReturn(VorgangListResponseTestFactory.create());
			when(statisticController.getVorgaengeStatistic()).thenReturn(StatisticTestFactory.create());
		}

		@DisplayName("Get all")
		@Nested
		class TestGetAll {

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT)))
						.andExpect(status().isOk());
			}
		}

		@DisplayName("Get all by status")
		@Nested
		class TestGetAllByStatus {

			private static final VorgangStatus STATUS = VorgangStatus.NEU;

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}

				@Test
				void shoulHaveSetStatus() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().getStatus()).hasValue(STATUS);
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_STATUS, STATUS.toString()))
						.andExpect(status().isOk());
			}
		}

		@DisplayName("Get all by searchBy")
		@Nested
		class TestGetAllBySearchBy {

			private static final String SEARCH_BY = "TestSearchByString";

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}

				@Test
				void shoulHaveSetSearchBy() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getSearchBy()).hasValue(SEARCH_BY);
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_SEARCH, SEARCH_BY))
						.andExpect(status().isOk());
			}
		}

		@DisplayName("Get all by assignedTo")
		@Nested
		class TestGetAllByAssigendTo {

			private static final UserId ASSIGNED_TO = UserProfileTestFactory.ID;

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}

				@Test
				void shoulHaveSetAssignedToAsOptional() {
					doRequest();

					verify(controller).asOptional(ASSIGNED_TO);
				}

				@Test
				void shoulHaveSetAssignedTo() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().getAssignedTo()).hasValue(ASSIGNED_TO);
				}

				@Test
				void shouldSetFilterByAssignedTo() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isFilterByAssignedTo()).isTrue();
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_ASSIGNED_TO, ASSIGNED_TO.toString()))
						.andExpect(status().isOk());
			}
		}

		@DisplayName("Get all by assignedTo and searchBy")
		@Nested
		class TestGetAllByAssigendToAndSearchBy {

			private static final UserId ASSIGNED_TO = UserProfileTestFactory.ID;
			private static final String SEARCH_BY = "TestSearchByString";

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}

				@Test
				void shouldSetAssignedAsOptional() {
					doRequest();

					verify(controller).asOptional(ASSIGNED_TO);
				}

				@Test
				void shoulHaveSetAssignedTo() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().getAssignedTo()).hasValue(ASSIGNED_TO);
				}

				@Test
				void shoulHaveSetSearchBy() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getSearchBy()).hasValue(SEARCH_BY);
				}

				@Test
				void shouldSetFilterByAssignedTo() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isFilterByAssignedTo()).isTrue();
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_ASSIGNED_TO, ASSIGNED_TO.toString())
								.param(VorgangController.PARAM_SEARCH, SEARCH_BY))
						.andExpect(status().isOk());
			}
		}

		@DisplayName("Get all by assignedTo and status")
		@Nested
		class TestGetAllByAssigendToAndStatus {

			private static final UserId ASSIGNED_TO = UserProfileTestFactory.ID;
			private static final VorgangStatus STATUS = VorgangStatus.NEU;

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}

				@Test
				void shouldSetAssignedAsOptional() {
					doRequest();

					verify(controller).asOptional(ASSIGNED_TO);
				}

				@Test
				void shoulHaveSetAssignedTo() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().getAssignedTo()).hasValue(ASSIGNED_TO);
				}

				@Test
				void shoulHaveSetStatus() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().getStatus()).hasValue(STATUS);
				}

				@Test
				void shouldSetFilterByAssignedTo() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isFilterByAssignedTo()).isTrue();
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_ASSIGNED_TO, ASSIGNED_TO.toString())
								.param(VorgangController.PARAM_STATUS, STATUS.toString()))
						.andExpect(status().isOk());
			}
		}

		@DisplayName("Get all by existing next wiedervorlage frist")
		@Nested
		class TestGetAllByExistingNextWiedervorlageFrist {

			private static final String HAS_NEXT_WIEDERVORLAGE_FRIST = "exists";

			@Test
			void shouldCallHandleGetAllRequest() {
				doRequest();

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@DisplayName("build criteria")
			@Nested
			class TestBuildCriteria {

				@Test
				void shouldHaveSetPage() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
				}

				@Test
				void shoulHaveSetLimit() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
				}

				@Test
				void shouldHaveCallExistNextWiedervorlageFrist() {
					doRequest();

					verify(controller).existNextWiedervorlageFrist(HAS_NEXT_WIEDERVORLAGE_FRIST);
				}

				@Test
				void shoulHaveSetHasNextWiedervorlageFrist() {
					doRequest();

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isHasNextWiedervorlageFrist()).isTrue();
				}
			}

			@SneakyThrows
			private void doRequest() {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_NEXT_WIEDERVORLAGE_FRIST, HAS_NEXT_WIEDERVORLAGE_FRIST))
						.andExpect(status().isOk());
			}

			@DisplayName("Get all by assignedTo and existing next wiedervorlage frist")
			@Nested
			class TestGetAllByAssignedtoAndExistingNextWiedervorlageFrist {

				private static final UserId ASSIGNED_TO = UserProfileTestFactory.ID;
				private static final String HAS_NEXT_WIEDERVORLAGE_FRIST = "exists";

				@Test
				void shouldCallHandleGetAllRequest() {
					doRequest();

					verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
				}

				@DisplayName("build criteria")
				@Nested
				class TestBuildCriteria {

					@Test
					void shouldHaveSetPage() {
						doRequest();

						verify(controller).handleGetAllRequest(criteriaCaptor.capture());
						assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
					}

					@Test
					void shoulHaveSetLimit() {
						doRequest();

						verify(controller).handleGetAllRequest(criteriaCaptor.capture());
						assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
					}

					@Test
					void shouldSetAssignedAsOptional() {
						doRequest();

						verify(controller).asOptional(ASSIGNED_TO);
					}

					@Test
					void shoulHaveSetAssignedTo() {
						doRequest();

						verify(controller).handleGetAllRequest(criteriaCaptor.capture());
						assertThat(criteriaCaptor.getValue().getFilterCriteria().getAssignedTo()).hasValue(ASSIGNED_TO);
					}

					@Test
					void shouldSetFilterByAssignedTo() {
						doRequest();

						verify(controller).handleGetAllRequest(criteriaCaptor.capture());
						assertThat(criteriaCaptor.getValue().getFilterCriteria().isFilterByAssignedTo()).isTrue();
					}

					@Test
					void shouldHaveCallExistNextWiedervorlageFrist() {
						doRequest();

						verify(controller).existNextWiedervorlageFrist(HAS_NEXT_WIEDERVORLAGE_FRIST);
					}

					@Test
					void shoulHaveSetHasNextWiedervorlageFrist() {
						doRequest();

						verify(controller).handleGetAllRequest(criteriaCaptor.capture());
						assertThat(criteriaCaptor.getValue().getFilterCriteria().isHasNextWiedervorlageFrist()).isTrue();
					}
				}

				@SneakyThrows
				private void doRequest() {
					mockMvc.perform(get(PATH)
									.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
									.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
									.param(VorgangController.PARAM_ASSIGNED_TO, ASSIGNED_TO.toString())
									.param(VorgangController.PARAM_NEXT_WIEDERVORLAGE_FRIST, HAS_NEXT_WIEDERVORLAGE_FRIST))
							.andExpect(status().isOk());
				}
			}
		}

		@Nested
		class TestGetAllByUngeleseneNachrichten {

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCallHandleGetAllRequest(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCriteriaHavePage(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(criteriaCaptor.capture());
				assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
			}

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCriteriaHaveLimit(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(criteriaCaptor.capture());
				assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
			}

			static List<String> nachrichtenValues() {
				return List.of(VorgangController.PARAM_NACHRICHTEN_UNGELESENE, "", "dummy");
			}

			@Nested
			class WithParamNachrichtenUngelesene {

				@Test
				void shouldCriteriaHaveHasUngeleseneNachrichtenTrue() {
					doRequestWithNachrichtenParam(VorgangController.PARAM_NACHRICHTEN_UNGELESENE);

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isHasUngeleseneNachrichten()).isTrue();
				}
			}

			@Nested
			class WithoutParamNachrichtenUngelesene {

				@ParameterizedTest
				@ValueSource(strings = {"", "dummy"})
				void shouldCriteriaHaveHasUngeleseneNachrichtenFalse(String nachrichtenValue) {
					doRequestWithNachrichtenParam(nachrichtenValue);

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isHasUngeleseneNachrichten()).isFalse();
				}
			}

			@SneakyThrows
			private void doRequestWithNachrichtenParam(String paramValue) {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_NACHRICHTEN, paramValue))
						.andExpect(status().isOk());
			}
		}

		@Nested
		class TestGetAllByAssignedToAndUngeleseneNachrichten {

			private static final UserId ASSIGNED_TO = UserProfileTestFactory.ID;

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCallHandleGetAllRequest(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(any(FindVorgaengeHeaderRequestCriteria.class));
			}

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCriteriaHavePage(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(criteriaCaptor.capture());
				assertThat(criteriaCaptor.getValue().getPage()).isEqualTo(PAGE);
			}

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCriteriaHaveLimit(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(criteriaCaptor.capture());
				assertThat(criteriaCaptor.getValue().getLimit()).isEqualTo(LIMIT);
			}

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldCriteriaHaveAssignedTo(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(criteriaCaptor.capture());
				assertThat(criteriaCaptor.getValue().getFilterCriteria().getAssignedTo()).hasValue(ASSIGNED_TO);
			}

			@ParameterizedTest
			@MethodSource("nachrichtenValues")
			void shouldSetFilterByAssignedTo(String nachrichtenValue) {
				doRequestWithNachrichtenParam(nachrichtenValue);

				verify(controller).handleGetAllRequest(criteriaCaptor.capture());
				assertThat(criteriaCaptor.getValue().getFilterCriteria().isFilterByAssignedTo()).isTrue();
			}

			static List<String> nachrichtenValues() {
				return List.of(VorgangController.PARAM_NACHRICHTEN_UNGELESENE, "", "dummy");
			}

			@Nested
			class WithParamNachrichtenUngelesene {

				@Test
				void shouldCriteriaHaveHasUngeleseneNachrichtenTrue() {
					doRequestWithNachrichtenParam(VorgangController.PARAM_NACHRICHTEN_UNGELESENE);

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isHasUngeleseneNachrichten()).isTrue();
				}
			}

			@Nested
			class WithoutParamNachrichtenUngelesene {

				@ParameterizedTest
				@ValueSource(strings = {"", "dummy"})
				void shouldCriteriaHaveHasUngeleseneNachrichtenFalse(String nachrichtenValue) {
					doRequestWithNachrichtenParam(nachrichtenValue);

					verify(controller).handleGetAllRequest(criteriaCaptor.capture());
					assertThat(criteriaCaptor.getValue().getFilterCriteria().isHasUngeleseneNachrichten()).isFalse();
				}
			}

			@SneakyThrows
			private void doRequestWithNachrichtenParam(String paramValue) {
				mockMvc.perform(get(PATH)
								.param(VorgangController.PARAM_PAGE, Integer.toString(PAGE))
								.param(VorgangController.PARAM_LIMIT, Integer.toString(LIMIT))
								.param(VorgangController.PARAM_ASSIGNED_TO, ASSIGNED_TO.toString())
								.param(VorgangController.PARAM_NACHRICHTEN, paramValue))
						.andExpect(status().isOk());
			}
		}
	}

	@DisplayName("As Optional")
	@Nested
	class TestAsOptional {

		@Test
		void shouldReturnEmptyOnEmptyUserId() {
			var optional = controller.asOptional(UserId.empty());

			assertThat(optional).isNotPresent();
		}

		@Test
		void shouldReturnFilledIfNotEmpty() {
			var userId = UserId.from("DummyUserId");

			var optional = controller.asOptional(userId);

			assertThat(optional).hasValue(userId);
		}
	}

	@DisplayName("Exist next wiedervorlage frist")
	@Nested
	class TestExistNextWiedervorlageFrist {

		@Test
		void shouldReturnTrueOnMatchingParameter() {
			var exists = controller.existNextWiedervorlageFrist(VorgangController.PARAM_NEXT_WIEDERVORLAGE_FRIST_EXISTS);

			assertThat(exists).isTrue();
		}

		@Test
		void shouldReturnFalseOnOther() {
			var exists = controller.existNextWiedervorlageFrist("quatsch");

			assertThat(exists).isFalse();
		}
	}

	@DisplayName("Vorgang with eingang")
	@Nested
	class TestVorgangWithEingang {

		private UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();

		@BeforeEach
		void initTest() {
			when(vorgangService.findVorgangWithEingang(any())).thenReturn(VorgangWithEingangTestFactory.create());
			when(modelAssembler.toModel(any())).then(i -> EntityModel.of(i.getArgument(0)));

			ApplicationContext context = mock(ApplicationContext.class);
			Environment environment = mock(Environment.class);
			when(environment.getProperty(anyString())).thenReturn("test/");
			when(context.getEnvironment()).thenReturn(environment);
			urlProvider.setApplicationContext(context);
		}

		@Test
		void shouldCallService() throws Exception {
			performRequest();

			verify(vorgangService).findVorgangWithEingang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			performRequest();

			verify(modelAssembler).toModel(any());
		}

		@Nested
		class TestDataMapping {

			@DisplayName("Single Field should be in json first level")
			@Test
			void singleField() throws Exception {
				performRequest().andExpect(jsonPath("$.eingang.formData." + SINGLE_FIELD_NAME).value(SINGLE_FIELD_VALUE));
			}

			@DisplayName("Numeric field should be numeric in json")
			@Test
			void numericField() throws Exception {
				performRequest().andExpect(jsonPath("$.eingang.formData." + NUMERIC_FIELD_NAME).isNumber());
				performRequest().andExpect(jsonPath("$.eingang.formData." + NUMERIC_FIELD_NAME).value(NUMERIC_FIELD_VALUE));
			}

			@DisplayName("SubForm should have name as key")
			@Test
			void keyOfsubForm() throws Exception {
				performRequest().andExpect(jsonPath("$.eingang.formData." + SUBFORM_NAME).isMap());
			}

			@DisplayName("SubForm-Field should be present")
			@Test
			void subFormField() throws Exception {
				performRequest().andExpect(jsonPath("$.eingang.formData." + SUBFORM_NAME + "." + SUBFORM_FIELD_NAME).value(SUBFORM_FIELD_VALUE));
			}

		}

		private ResultActions performRequest() throws Exception {
			return mockMvc.perform(get(PATH + "/" + VorgangHeaderTestFactory.ID)).andExpect(status().isOk());
		}
	}

	@DisplayName("Reset new postfachNachricht")
	@Nested
	class TestResetNewPostfachNachricht {

		@Test
		void resetNewPostfachNachricht() throws Exception {
			performRequest("{\"hasNewPostfachNachricht\":false}").andExpect(status().isNoContent());

			verify(clientAttributeService).resetPostfachNachricht(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldReturnUnprocessableEntity() throws Exception {
			performRequest("{\"hasNewPostfachNachricht\":true}").andExpect(status().isUnprocessableEntity());

			verify(clientAttributeService, never()).resetPostfachNachricht(VorgangHeaderTestFactory.ID);
		}

		private ResultActions performRequest(String body) throws Exception {
			return mockMvc
					.perform(put(RESET_POSTFACH_NACHRICHT_PATH, VorgangHeaderTestFactory.ID).contentType(MediaType.APPLICATION_JSON_VALUE)
							.content(body));
		}
	}
}