package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;
import java.util.Optional;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.MockedStatic;
import org.mockito.Mockito;

import de.ozgcloud.common.errorhandling.TechnicalException;

class DateConverterTest {

	@Nested
	class TestConvertGermanFormatToISO {

		private static final int YEAR = 2023;
		private static final int MONTH = 2;
		private static final int DAY = 14;

		private static final String DATE = String.format("%s.%s%s.%s", DAY, 0, MONTH, YEAR);

		@Test
		void shouldConvert() {
			var converted = callConverter(DATE);

			assertThat(converted).get().extracting(XMLGregorianCalendar::getYear, XMLGregorianCalendar::getMonth, XMLGregorianCalendar::getDay)
					.containsExactly(YEAR, MONTH, DAY);
		}

		@ParameterizedTest
		@ValueSource(strings = { "2005-10-10", "abc", "19. February 1970", " ", "" })
		void shouldReturnEmpty(String dateStr) {
			var converted = callConverter(dateStr);

			assertThat(converted).isEmpty();
		}

		@Test
		void shouldReturnEmptyIfDateIsNull() {
			var converted = callConverter(null);

			assertThat(converted).isEmpty();
		}

		private Optional<XMLGregorianCalendar> callConverter(String dateStr) {
			return DateConverter.convertGermanFormatToISO(dateStr);
		}
	}

	@Nested
	class TestCreateDatatypeFactory {

		@Test
		void shouldThrowTechnicalException() {
			try (MockedStatic<DatatypeFactory> datatypeFactory = Mockito.mockStatic(DatatypeFactory.class)) {
				datatypeFactory.when(DatatypeFactory::newInstance).thenThrow(DatatypeConfigurationException.class);

				assertThatThrownBy(() -> DateConverter.createDatatypeFactory()).isInstanceOf(TechnicalException.class);
			}
		}
	}

	@Nested
	class TestToXmlGregorianCalendar {

		@Test
		void shouldConvertZonedDateTime() {
			var date = ZonedDateTime.now();

			var xmlGregorianCalendar = DateConverter.toXmlGregorianCalendar(date);

			assertThat(xmlGregorianCalendar.getYear()).isEqualTo(date.getYear());
			assertThat(xmlGregorianCalendar.getMonth()).isEqualTo(date.getMonthValue());
			assertThat(xmlGregorianCalendar.getDay()).isEqualTo(date.getDayOfMonth());
		}

	}
}